Option Compare Binary
Option Explicit On 
Option Strict On

Imports System
Imports System.IO
Imports System.Xml
Imports System.Xml.Serialization

''' <summary>
'''   Stellt Funktionalitt zum Zugriff auf bestimmte Einstellungen, die aus einer Datei geladen
'''   bzw. in eine Datei gespeichert werden knnen, bereit.
''' </summary>
Public Class Configuration
    Private m_Configuration As ConfigurationData
    Private m_DefaultConfiguration As ConfigurationData

    ''' <summary>
    '''   Erstellt eine neue Instanz von <c>Configuration</c> und weist den Eigenschaften die
    '''   in den Parametern angegebenen Werte zu.
    ''' </summary>
    ''' <param name="DefaultData">Wert der Eigenschaft <c>DefaultData</c>.</param>
    Public Sub New(ByVal DefaultData As ConfigurationData)
        Me.Configuration = DefaultData
        Me.DefaultConfiguration = DefaultData
    End Sub

    ''' <summary>
    '''   Speichert die Einstellungen in die im Parameter <paramref name="FileName"/>
    '''   angegebene Datei.
    ''' </summary>
    ''' <param name="FileName">Name der Datei, in die die Einstellungen gespeichert werden.</param>
    ''' <exception cref="System.Exception">Wird ausgelst, wenn beim Speichern ein Fehler auftritt.</exception>
    Public Sub Save(ByVal FileName As String)
        Dim writer As XmlWriter
        Try
            Dim ser As XmlSerializer = New XmlSerializer(GetType(ConfigurationData))
            writer = New XmlTextWriter(FileName, New System.Text.UTF8Encoding)
            ser.Serialize(writer, m_Configuration)
        Catch ex As Exception
            Throw ex
        Finally
            writer.Close()
        End Try
        writer.Close()
    End Sub

    ''' <summary>
    '''   Ldt die Einstellungen aus der im Parameter <paramref name="FileName"/> angegebenen
    '''   Datei.
    ''' </summary>
    ''' <param name="FileName">Name der Datei, die die Einstellungen enthlt.</param>
    ''' <exception cref="System.Exception">
    '''   Wird ausgelst, wenn ein Fehler beim Laden auftritt und daher die Standard-Einstellungen
    '''   verwendet werden.
    ''' </exception>
    Public Sub Load(ByVal FileName As String)
        Try
            Dim ser As XmlSerializer = New XmlSerializer(GetType(ConfigurationData))
            Dim fs As FileStream = New FileStream(FileName, FileMode.Open)
            Dim reader As XmlReader = New XmlTextReader(fs)
            m_Configuration = CType(ser.Deserialize(reader), ConfigurationData)
            reader.Close()
        Catch ex As Exception

            ' Es ist ein Fehler beim Lesen aufgetreten (Datei existiert nicht, kann nicht gelesen
            ' werden oder besitzt ein ungltiges Format, das nicht deserialsiert werden kann).
            ' In diesem Fall werden die im Konstruktor der Klasse angegebenen Standardeinstellungen
            ' benutzt.
            m_Configuration = m_DefaultConfiguration
            Throw New Exception("Error loading configuration, using default configuration data instead.", ex)
        End Try
    End Sub

    ''' <summary>
    '''   Gibt die Konfigurationseinstellungen an oder gibt sie zurck.
    ''' </summary>
    ''' <value>Objekt, das die Konfigurationseinstellungen enthlt.</value>
    Public Property Configuration() As ConfigurationData
        Get
            Return m_Configuration
        End Get
        Set(ByVal Value As ConfigurationData)
            m_Configuration = Value
        End Set
    End Property

    ''' <summary>
    '''   Gibt die Standard-Konfigurationseinstellungen an oder gibt sie zurck.
    ''' </summary>
    ''' <value>Objekt, das die Standard-Konfigurationseinstellungen enthlt.</value>
    Public Property DefaultConfiguration() As ConfigurationData
        Get
            Return m_DefaultConfiguration
        End Get
        Set(ByVal Value As ConfigurationData)
            m_DefaultConfiguration = Value
        End Set
    End Property
End Class